@@ -21,6 +21,7 @@ import (
 	"os"
 
 	"github.com/fusor/mig-controller/pkg/apis"
+	"github.com/fusor/mig-controller/pkg/compat/conversion"
 	"github.com/fusor/mig-controller/pkg/controller"
 	"github.com/fusor/mig-controller/pkg/webhook"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
@@ -74,6 +75,10 @@ func main() {
 		log.Error(err, "unable add Cluster Registry APIs to scheme")
 		os.Exit(1)
 	}
+	if err := conversion.RegisterConversions(mgr.GetScheme()); err != nil {
+		log.Error(err, "unable to register nessesary conversions")
+		os.Exit(1)
+	}
 
 	// Setup all Controllers
 	log.Info("Setting up controller")
