@@ -28,16 +28,20 @@ import (
 
 	pvdr "github.com/fusor/mig-controller/pkg/cloudprovider"
 	migref "github.com/fusor/mig-controller/pkg/reference"
+	"github.com/fusor/mig-controller/pkg/settings"
 	appsv1 "github.com/openshift/api/apps/v1"
 	imagev1 "github.com/openshift/api/image/v1"
 	velero "github.com/vmware-tanzu/velero/pkg/apis/velero/v1"
 	kapi "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	k8sLabels "k8s.io/apimachinery/pkg/labels"
 	"k8s.io/apimachinery/pkg/util/intstr"
 	k8sclient "sigs.k8s.io/controller-runtime/pkg/client"
 )
 
+var Settings = &settings.Settings
+
 // Cache Indexes.
 const (
 	ClosedIndexField = "closed"
@@ -354,7 +358,7 @@ func (r *MigPlan) EqualsRegistryImageStream(a, b *imagev1.ImageStream) bool {
 }
 
 // Build a Registry DeploymentConfig as desired for the source cluster.
-func (r *MigPlan) BuildRegistryDC(storage *MigStorage, name, dirName string) (*appsv1.DeploymentConfig, error) {
+func (r *MigPlan) BuildRegistryDC(storage *MigStorage, proxySecret *kapi.Secret, name, dirName string) (*appsv1.DeploymentConfig, error) {
 	labels := r.GetCorrelationLabels()
 	labels[MigrationRegistryLabel] = string(r.UID)
 	labels["app"] = name
@@ -365,12 +369,55 @@ func (r *MigPlan) BuildRegistryDC(storage *MigStorage, name, dirName string) (*a
 			Namespace: VeleroNamespace,
 		},
 	}
-	err := r.UpdateRegistryDC(storage, deploymentconfig, name, dirName)
+	err := r.UpdateRegistryDC(storage, deploymentconfig, proxySecret, name, dirName)
 	return deploymentconfig, err
 }
 
+// Get registry proxy secret for registry DC
+// Returns nil if secret isn't found or no configuration exists
+func (r *MigPlan) GetRegistryProxySecret(client k8sclient.Client) (*kapi.Secret, error) {
+	list := kapi.SecretList{}
+	selector := k8sLabels.SelectorFromSet(map[string]string{
+		"migration-proxy-config": "true",
+	})
+	err := client.List(
+		context.TODO(),
+		&k8sclient.ListOptions{
+			Namespace:     VeleroNamespace,
+			LabelSelector: selector,
+		},
+		&list,
+	)
+	// If error listing secrets return no env vars
+	if err != nil {
+		return nil, err
+	}
+	// If no secrets found, return nil and no error
+	// This means no proxy is configured
+	if len(list.Items) == 0 {
+		return nil, nil
+	}
+	if len(list.Items) > 1 {
+		return nil, errors.New("found multiple proxy config secrets")
+	}
+
+	return &list.Items[0], nil
+}
+
 // Update a Registry DeploymentConfig as desired for the specified cluster.
-func (r *MigPlan) UpdateRegistryDC(storage *MigStorage, deploymentconfig *appsv1.DeploymentConfig, name, dirName string) error {
+func (r *MigPlan) UpdateRegistryDC(storage *MigStorage, deploymentconfig *appsv1.DeploymentConfig, proxySecret *kapi.Secret, name, dirName string) error {
+	envFrom := []kapi.EnvFromSource{}
+	// If Proxy secret exists, set env from it
+	if proxySecret != nil {
+		source := kapi.EnvFromSource{
+			SecretRef: &kapi.SecretEnvSource{
+				LocalObjectReference: kapi.LocalObjectReference{
+					Name: proxySecret.Name,
+				},
+			},
+		}
+		envFrom = append(envFrom, source)
+	}
 	deploymentconfig.Spec = appsv1.DeploymentConfigSpec{
 		Replicas: 1,
 		Selector: map[string]string{
@@ -389,8 +436,9 @@ func (r *MigPlan) UpdateRegistryDC(storage *MigStorage, deploymentconfig *appsv1
 			Spec: kapi.PodSpec{
 				Containers: []kapi.Container{
 					kapi.Container{
-						Image: migRegistryImageRef(),
-						Name:  "registry",
+						EnvFrom: envFrom,
+						Image:   migRegistryImageRef(),
+						Name:    "registry",
 						Ports: []kapi.ContainerPort{
 							kapi.ContainerPort{
 								ContainerPort: 5000,
