@@ -20,8 +20,8 @@ import (
 	"github.com/fusor/mig-controller/pkg/controller/remotewatcher"
 	"github.com/fusor/mig-controller/pkg/imagescheme"
 	"github.com/fusor/mig-controller/pkg/remote"
-	velerov1 "github.com/heptio/velero/pkg/apis/velero/v1"
 	appsv1 "github.com/openshift/api/apps/v1"
+	velerov1 "github.com/vmware-tanzu/velero/pkg/apis/velero/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"sigs.k8s.io/controller-runtime/pkg/event"
 	"sigs.k8s.io/controller-runtime/pkg/handler"
